/**
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Wei&szlig;enfelser Stra&szlig;e 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package org.geotools.data.vrz3;

import org.geotools.feature.AttributeType;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DoubleAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.ReferenceAttributeType;
import de.bsvrz.dav.daf.main.config.StringAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.TimeAttributeType;

/**
 * Feature-Attribut eines VRZ3-Features
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: VRZ3FeatureAttribute.java 6072 2008-01-30 17:11:25Z gieseler $
 *
 */

public class VRZ3FeatureAttribute {
	
	/** der zugeh&ouml;rige AttributeType der GeoTools-Library */
	private AttributeType _geotoolsAttributeType = null;
	
	/** DaV-Attributgruppe	*/
	private String _davatg = null;
	
	/** DaV-Aspekt	*/
    private String _davasp = null;
    
    /** DaV-Attribut	*/
    private String _davatt = null;
    
	//private ListAttributeType lat;
	
	/** der ValueSetter */
	private VRZ3ValueSetter _valueSetter = null;
	
	/**
	 * @param atg Attributgruppe
	 * @param name Attribut
	 */
	public VRZ3FeatureAttribute(String atg, String name) {
		_davatg = atg;
		_davatt = name;
	}


	/**
	 * @return the _geotoolsAttributeType
	 */
	public AttributeType get_geotoolsAttributeType() {
		return _geotoolsAttributeType;
	}

	/**
	 * @param attributeType the _geotoolsAttributeType to set
	 */
	public void set_geotoolsAttributeType(AttributeType attributeType) {
		_geotoolsAttributeType = attributeType;
	}

//	/**
//	 * @return
//	 */
//	public List<AttributeType> bestimmeAttribute(ClientDavConnection dav) {
//    	Data data = createData(dav, _davatg, _davasp);
//    	List<AttributeType> attlist = new ArrayList<AttributeType>();
//    	
//		if (data.isPlain()) {
//			attlist.add(getAttribute(data));
//		} else if (data.isArray()) {
//			/* Iterator zum Durchlaufen des Feldes */
//			Iterator i = data.iterator();
////
////			/* Name dieses Datums */
//////			String name = data.getName();
////
////			while (i.hasNext()) {
////				atts.add(getAttribute((Data) i.next()));
////			}
//		} else if (data.isList()) {
//			/* Iterator zum Durchlaufen der Liste */
//			Iterator i = data.iterator();
//
//			/* Name dieses Datums */
//			String name = data.getName();
//
//			List<AttributeType> listlist = new ArrayList<AttributeType>();
//			while (i.hasNext()) {
//				listlist.add(getAttribute((Data) i.next()));
//			}
//			
////			lat = new ListAttributeType(name, false, listlist.toArray(new AttributeType[listlist.size()]));
////			attlist.add(lat);
//			
//			attlist.addAll(listlist);
//		}
//		
//		return attlist;
//	}
//	
	
//	AttributeType getAttribute( Data data ) {
//        Class<?> typeClass;
//		String name = data.getName();
//		String defaultvalue = null;
//		de.bsvrz.dav.daf.main.config.AttributeType type = data.getAttributeType();
//	
//		if (type instanceof StringAttributeType) {
//			typeClass = String.class;
//		} else if (type instanceof IntegerAttributeType) {
//			typeClass = Integer.class;
//			
//			IntegerValueRange wertebereich = ((IntegerAttributeType) type).getRange();
//			defaultvalue = ((IntegerAttributeType) type).getDefaultAttributeValue();
//			_valueSetter = new VRZ3ValueSetter(defaultvalue) {
//                protected void stringToValue()
//                    throws Exception {
//                    	objValue = new Integer(strValue);
//                }
//            };
//		} else if (type instanceof DoubleAttributeType) {
//			typeClass = Double.class;
//		} else if (type instanceof TimeAttributeType) {
//			typeClass = Integer.class;
//		} else if (type instanceof ReferenceAttributeType) {
//			typeClass = String.class;
////			result = new ObjektReferenz(data.getName());
////			SystemObject ref = data.asReferenceValue().getSystemObject();
////			if (ref == null) {
////				((ObjektReferenz) result).set(0L);
////			} else {
////				((ObjektReferenz) result).set(ref.getId());
////			}
//		}
//		else
//			return null;
//		
////		AttributeType at = AttributeTypeFactory.newAttributeType(name,
////            typeClass, false, size, defaultvalue);
//		
//		AttributeType at = AttributeTypeFactory.newAttributeType(name,
//	            typeClass, false, 0, defaultvalue);
//		
//		return at;
//	}
	
//	private Data createData( ClientDavConnection dav, String atg, String asp ) {
//		DataModel dm = dav.getDataModel();	
//	
//    	AttributeGroup attributeGroup = (AttributeGroup) dav.getDataModel().getAttributeGroup(atg);
//    	Aspect aspect = dav.getDataModel().getAspect(asp);
//    	DataDescription dataDescription = new DataDescription(attributeGroup, aspect);
//
//    	return dav.createData( attributeGroup );
//	}

	/**
	 * @return the _davatg
	 */
	public String get_davatg() {
		return _davatg;
	}

	/**
	 * @return the _davasp
	 */
	public String get_davasp() {
		return _davasp;
	}

	/**
	 * Gibt den Wert des Attributes zur&uuml;ck.
	 * 
	 * @param so Systemobjekt 
	 * @return Wert des Attributes
	 */
	public Object getAttributWert(SystemObject so) {
//	   	Data data = createData(dav, _davatg, _davasp);
    	//List<Object> attlist = new ArrayList<Object>();
    	
    	if(_davatg.equals("Name")) {
			_valueSetter.setString(so.getName());
			return _valueSetter.getValue();
    	}
    	
    	if(_davatg.equals("PID")) {
			_valueSetter.setString(so.getPid());
			return _valueSetter.getValue();
    	}
    	
   		AttributeGroup ratg = so.getDataModel().getAttributeGroup(_davatg);
		Data sodata = so.getConfigurationData(ratg);
		
		Data data = sodata.getItem(_davatt);
		de.bsvrz.dav.daf.main.config.AttributeType type = data.getAttributeType();
//		String ots = data.toString();
//		ots = data.valueToString();
		
		if (type instanceof StringAttributeType) {
			if( data.isArray())
				_valueSetter.setString(data.asTextArray().toString());
			else
				_valueSetter.setString(data.asTextValue().toString());
		} else if (type instanceof IntegerAttributeType) {
			_valueSetter.setString(data.asScaledValue().getText());
//			IntegerValueRange range = ((IntegerAttributeType) type).getRange();
//			if(range.getConversionFactor() != 0.0) {
//				_valueSetter.setString(""+data.asScaledValue().doubleValue());
//			}
//			else {
//				_valueSetter.setString(""+data.asUnscaledValue().intValue());
//			}
		} else if (type instanceof DoubleAttributeType) {
			_valueSetter.setString(data.asScaledValue().toString());
		} else if (type instanceof TimeAttributeType) {
			_valueSetter.setString(data.asScaledValue().toString());
		} else if (type instanceof ReferenceAttributeType) {
			_valueSetter.setString(data.asTextValue().getText());
//			result = new ObjektReferenz(data.getName());
//			SystemObject ref = data.asReferenceValue().getSystemObject();
//			if (ref == null) {
//				((ObjektReferenz) result).set(0L);
//			} else {
//				((ObjektReferenz) result).set(ref.getId());
//			}
		}
		else
			return null;		

		return _valueSetter.getValue();
	}

	
//	/**
//	 * @param data
//	 * @return
//	 */
//	private Object getAttributWert(Data data) {
//		_valueSetter.setValue(data.getItem(_davatt).asTextValue());
//		return _valueSetter.getValue();
//	}


	/**
	 * @return
	 */
	public AttributeType getAttibuteType() {
		return null;
	}


	/**
	 * @param valueSetter
	 */
	public void setValueSetter(VRZ3ValueSetter valueSetter) {
		_valueSetter = valueSetter;
	}
	
}
